from tcra import fragility
from tcra import simulation

__version__ = '0.1.2'

__copyright__ = """Copyright 2015 Sandia Corporation.
Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
the U.S. Government retains certain rights in this software."""

__license__ = "Revised BSD License"
